<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://modeltheme.com/
 * @since      1.0.0
 *
 * @package    Mtkb
 * @subpackage Mtkb/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Mtkb
 * @subpackage Mtkb/includes
 * @author     ModelTheme <support@modeltheme.com>
 */
class Mtkb {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Mtkb_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */

	/**
	 * Store plugin admin class to allow public access.
	 *
	 * @since    20180622
	 * @var object      The admin class.
	 */
	public $admin;

	/**
	 * Store plugin admin class to allow public access.
	 *
	 * @since    20180622
	 * @var object      The admin class.
	 */
	public $shortcode_manager;

	/**
	 * Store plugin public class to allow public access.
	 *
	 * @since    20180622
	 * @var object      The admin class.
	 */
	public $public;

	/**
	 * Store plugin public class to allow public access.
	 *
	 * @since    20180622
	 * @var object      The admin class.
	 */
	public $wp_bakery_map;
	

	public function __construct() {
		if ( defined( 'MTKB_VERSION' ) ) {
			$this->version = MTKB_VERSION;
		} else {
			$this->version = '1.2.2';
		}
		$this->plugin_name = 'mtkb';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_shortcodes_manager_hooks();
		$this->define_public_hooks();
		$this->define_wp_bakery_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Mtkb_Loader. Orchestrates the hooks of the plugin.
	 * - Mtkb_i18n. Defines internationalization functionality.
	 * - Mtkb_Admin. Defines all hooks for the admin area.
	 * - Mtkb_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-mt-knowledgebase-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-mt-knowledgebase-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-mt-knowledgebase-admin.php';

		/**
		 * The class responsible for displaying the shortcode manager in admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-mt-knowledgebase-shortcode-manager.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-mt-knowledgebase-public.php';

		/**
		 * The class responsible for mapping the shortcodes to Wp Bakery
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-mt-knowledgebase-wp-bakery.php';

		/**
		 * The class responsible for including cmb2 library
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/cmb2/init.php';

		/**
		 * The class responsible for mapping the shortcodes to elementor
		 */

		include_once(ABSPATH.'wp-admin/includes/plugin.php');
		if (is_plugin_active( 'elementor/elementor.php' )) {
    		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-mt-knowledgebase-elementor.php';
	    }


		$this->loader = new Mtkb_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Mtkb_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Mtkb_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

    	$this->admin = new Mtkb_Admin( $this->get_plugin_name(), $this->get_version() );


		$this->loader->add_action( 'admin_enqueue_scripts', $this->admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $this->admin, 'enqueue_scripts' );
		//admin pages/ options
		$this->loader->add_action( 'admin_init', $this->admin, 'mtkb_register_settings' );
 		$this->loader->add_action( 'admin_menu', $this->admin, 'mtkb_add_menu_page' );
		$this->loader->add_action( 'admin_menu', $this->admin, 'mtkb_add_submenu_pages' );
		
		
		if ((get_option("mtkb_knowledgebase_enabled") == "yes")) {

			//CPT
			$this->loader->add_action( 'init', $this->admin, 'mtkb_register_knowledgebase_custom_post' );
			//CPT - TAX
			$this->loader->add_action( 'init', $this->admin, 'mtkb_register_knowledgebase_category' );
			$this->loader->add_action( 'init', $this->admin, 'mtkb_register_knowledgebase_tags' );

		}
		if ((get_option("mtkb_changelog_enabled") == "yes")) {

			//CPT
			$this->loader->add_action( 'init', $this->admin, 'mtkb_register_changelog_custom_post' );

		}
		if ((get_option("mtkb_portfolio_enabled") == "yes")) {

			//CPT
			$this->loader->add_action( 'init', $this->admin, 'mtkb_register_portfolio_custom_post' );
			//CPT - TAX
			$this->loader->add_action( 'init', $this->admin, 'mtkb_register_portfolio_category' );
			$this->loader->add_action( 'init', $this->admin, 'mtkb_register_portfolio_tags' );
		    $this->loader->add_action( 'add_meta_boxes', $this->admin, 'mtkb_portfolio_add_meta_box' );
		    $this->loader->add_action( 'save_post', $this->admin, 'mtkb_portfolio_save_page_meta_box' );

		}
	    /* changelog repeatable metabox */
			$this->loader->add_action( 'cmb2_admin_init', $this->admin, 'mtkb_changelog_repeatable_wysiwyg' );
		
	}

	/**
	 * Register all of the hooks related to the shortcode manager functionality
	 * of the plugin.
	 *
	 * @since    1.1.1
	 * @access   private
	 */
	private function define_shortcodes_manager_hooks() {

    	$this->shortcode_manager = new Mtkb_Shortcode_Manager( $this->get_plugin_name(), $this->get_version() );

    	//shortcode manager
		$this->loader->add_action('admin_enqueue_scripts', $this->shortcode_manager, 'mtkb_shortcode_builder_enqueue_admin_scripts');
		$this->loader->add_action( 'init', $this->shortcode_manager, 'mtkb_shortcode_builderbuttons' );
		$this->loader->add_action( 'in_admin_header', $this->shortcode_manager,'mtkb_shortcode_buildermodal_shortcode_html' );
    }

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

	    $this->public = new Mtkb_Public( $this->get_plugin_name(), $this->get_version() );

	    //actions
			$this->loader->add_action( 'wp_enqueue_scripts', $this->public, 'enqueue_styles' );
			$this->loader->add_action( 'wp_enqueue_scripts', $this->public, 'enqueue_scripts' );
			$this->loader->add_action( 'wp_kses_allowed_html', $this->public, 'mtkb_kses_allowed_html', 10, 2); 
		
		//Knowledgebase
		if (get_option("mtkb_knowledgebase_enabled") == "yes") {
	    	//actions
			$this->loader->add_action( 'wp_ajax_data_fetch' , $this->public, 'mtkb_search_data_fetch');
			$this->loader->add_action( 'wp_ajax_nopriv_data_fetch', $this->public,'mtkb_search_data_fetch');
			$this->loader->add_action( 'wp_ajax_mtkb_love_post', $this->public, 'mtkb_love_post'); 
			$this->loader->add_action( 'wp_ajax_nopriv_mtkb_love_post', $this->public, 'mtkb_love_post'); 
			$this->loader->add_action( 'widgets_init', $this->public, 'mtkb_register_sidebar' );
			$this->loader->add_action( 'after_setup_theme', $this->public, 'mtkb_portfolio_thumbnail_size' );
	    	//filters
		    $this->loader->add_filter( 'template_include', $this->public , 'mtkb_template_from_directory_knowledgebase' );
		    $this->loader->add_filter( 'template_redirect', $this->public, 'mtkb_views_count' );

		    //shortcodes
		    $this->loader->add_shortcode( 'mt_knowledgebase_grid_container', $this->public, 'mtkb_shortcode_knowledge_grid_container' );
		    $this->loader->add_shortcode( 'mt_knowledgebase_list', $this->public, 'mtkb_shortcode_list' );
		    $this->loader->add_shortcode( 'mt_knowledgebase_most', $this->public, 'mtkb_shortcode_most' );
		    $this->loader->add_shortcode( 'mt_knowledgebase_search', $this->public, 'mtkb_shortcode_ico_search' );
		    $this->loader->add_shortcode( 'mt_knowledgebase_category_grid', $this->public, 'mtkb_shortcode_categories_grid' );
		    $this->loader->add_shortcode( 'mt_knowledgebase_list_a_z', $this->public, 'mtkb_shortcode_knowledge_a_z' );
		    $this->loader->add_shortcode( 'mt_knowledgebase_category_accordion', $this->public, 'mtkb_shortcode_knowledgebase_category_accordion' );
		    $this->loader->add_shortcode( 'mt_knowledgebase_categories_accordion', $this->public, 'mtkb_shortcode_categories_accordion' );

		}
		// Changelogs
		if (get_option("mtkb_changelog_enabled") == "yes") {
			//actions
			//filters
    		$this->loader->add_filter( 'single_template', $this->public, 'mtkb_taxonomy_template_from_directory_changelog' );

		    //shortcodes
		    $this->loader->add_shortcode( 'mt_changelog_container', $this->public, 'mtkb_shortcode_changelog_container' );
		    $this->loader->add_shortcode( 'mt_changelog_item', $this->public, 'mtkb_shortcode_changelog_container_item' );
		    $this->loader->add_shortcode( 'mt_single_changelog', $this->public, 'mtkb_shortcode_single_changelog' );
		}
		
		//Faqs
	    $this->loader->add_shortcode( 'mt_faqs_container', $this->public, 'mtkb_shortcode_faqs_container' );
	    $this->loader->add_shortcode( 'mt_faqs_item', $this->public, 'mtkb_shortcode_faqs_item' );

	    //Knowledgebase
		if (get_option("mtkb_portfolio_enabled") == "yes") {

			//filters
	    	$this->loader->add_filter( 'single_template', $this->public, 'mtkb_single_template_from_directory_portfolio' );
	    	$this->loader->add_filter( 'taxonomy_template', $this->public, 'mtkb_taxonomy_template_from_directory_portfolio' );
	    	$this->loader->add_action( 'mtkb-portfolio-share', $this->public, 'mtkb_portfolios_share_buttons' );
	    	//shortcodes
		    $this->loader->add_shortcode( 'mt_portfolio_filters', $this->public, 'mtkb_portfolio_shortcode_filters' );
		    $this->loader->add_shortcode( 'mt_portfolio_category', $this->public, 'mtkb_portfolio_shortcode_category' );
		    $this->loader->add_shortcode( 'mt_portfolio_most', $this->public, 'mtkb_shortcode_most_portfolio' );
		}


	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_wp_bakery_hooks() {

	    $this->wp_bakery_map = new Mtkb_Wp_Bakery_Map( $this->get_plugin_name(), $this->get_version() );
		if (get_option("mtkb_knowledgebase_enabled") == "yes") {
			$this->loader->add_action('init', $this->wp_bakery_map, 'mtkb_vc_shortcodes'); 
		}

		if (get_option("mtkb_changelog_enabled") == "yes") {
			$this->loader->add_action('init', $this->wp_bakery_map, 'mtkb_changelog_vc_shortcodes'); 
		}

		if (get_option("mtkb_portfolio_enabled") == "yes") {
			$this->loader->add_action('init', $this->wp_bakery_map, 'mtkb_portfolio_vc_shortcodes'); 
		}

		$this->loader->add_action('init', $this->wp_bakery_map, 'mtkb_faqs_vc_shortcodes'); 
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Mtkb_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
